@@ -14,7 +14,7 @@ __author__ = (
     "Mark Harviston <mark.harviston@gmail.com>, "
     "Arve Knudsen <arve.knudsen@gmail.com>",
 )
-__version__ = "0.23.0"
+__version__ = "0.24.0"
 __url__ = "https://github.com/CabbageDevelopment/qasync"
 __license__ = "BSD"
 __all__ = ["QEventLoop", "QThreadExecutor", "asyncSlot", "asyncClose"]
@@ -23,6 +23,7 @@ import asyncio
 import contextlib
 import functools
 import importlib
+import inspect
 import itertools
 import logging
 import os
@@ -184,11 +185,11 @@ class QThreadExecutor:
         if stack_size is None:
             # Match cpython/Python/thread_pthread.h
             if sys.platform.startswith("darwin"):
-                stack_size = 16 * 2 ** 20
+                stack_size = 16 * 2**20
             elif sys.platform.startswith("freebsd"):
-                stack_size = 4 * 2 ** 20
+                stack_size = 4 * 2**20
             elif sys.platform.startswith("aix"):
-                stack_size = 2 * 2 ** 20
+                stack_size = 2 * 2**20
         self.__workers = [
             _QThreadWorker(self.__queue, i + 1, stack_size) for i in range(max_workers)
         ]
@@ -373,6 +374,7 @@ class _QEventLoop:
 
         try:
             self.__log_debug("Starting Qt event loop")
+            asyncio.events._set_running_loop(self)
             rslt = -1
             try:
                 rslt = self.__app.exec_()
@@ -381,6 +383,7 @@ class _QEventLoop:
             self.__log_debug("Qt event loop ended with result %s", rslt)
             return rslt
         finally:
+            asyncio.events._set_running_loop(None)
             self._after_run_forever()
             self.__is_running = False
 
@@ -785,8 +788,27 @@ def asyncSlot(*args, **kwargs):
         @Slot(*args, **kwargs)
         @functools.wraps(fn)
         def wrapper(*args, **kwargs):
-            task = asyncio.ensure_future(fn(*args, **kwargs))
-            task.add_done_callback(_error_handler)
+            # Qt ignores trailing args from a signal but python does
+            # not so inspect the slot signature and if it's not
+            # callable try removing args until it is.
+            task = None
+            while len(args):
+                try:
+                    inspect.signature(fn).bind(*args, **kwargs)
+                except TypeError:
+                    if len(args):
+                        # Only convert args to a list if we need to pop()
+                        args = list(args)
+                        args.pop()
+                        continue
+                else:
+                    task = asyncio.ensure_future(fn(*args, **kwargs))
+                    task.add_done_callback(_error_handler)
+                    break
+            if task is None:
+                raise TypeError(
+                    "asyncSlot was not callable from Signal. Potential signature mismatch."
+                )
             return task
 
         return wrapper
@@ -796,7 +818,7 @@ def asyncSlot(*args, **kwargs):
 
 class QEventLoopPolicyMixin:
     def new_event_loop(self):
-        return QEventLoop(QApplication(sys.argv))
+        return QEventLoop(QApplication.instance() or QApplication(sys.argv))
 
 
 class DefaultQEventLoopPolicy(
